#ifndef __BASIC4GLPLUGIN__
#define __BASIC4GLPLUGIN__

#define BASIC4GL_DLL_VERSION 3

' Type constants
enum DLL_Basic4GL_TypeCode
  DLL_BASIC4GL_INT    = 0
  DLL_BASIC4GL_FLOAT
  DLL_BASIC4GL_STRING
end enum

enum DLL_Basic4GL_Ext_TypeCode
  DLL_BASIC4GL_EXT_PADDING = -256 ' Empty padding in C structure
  DLL_BASIC4GL_EXT_BYTE
  DLL_BASIC4GL_EXT_WORD
  DLL_BASIC4GL_EXT_INT
  DLL_BASIC4GL_EXT_INT64  ' 64 bit integer
  DLL_BASIC4GL_EXT_FLOAT  ' 32 bit floating point number
  DLL_BASIC4GL_EXT_DOUBLE ' 64 bit floating point number
  DLL_BASIC4GL_EXT_STRING ' Fixed length string
end enum

#define THIS_PLUGIN      byref _plugin      as IDLL_Basic4GL_Plugin ptr
#define THIS_REGISTRY    byref _registry    as IDLL_Basic4GL_FunctionRegistry ptr
#define THIS_RUNTIME     byref _runtime     as IDLL_Basic4GL_Runtime ptr
#define THIS_COMPILER    byref _compiler    as IB4GLCompiler ptr
#define THIS_OBJECTSTORE byref _objectstore as IDLL_Basic4GL_ObjectStore ptr

' register int,float,string,type,void function
#define RIF(n,a)   _registry->RegisterFunction(_registry,#n,a,DLL_BASIC4GL_INT)
#define RFF(n,a)   _registry->RegisterFunction(_registry,#n,a,DLL_BASIC4GL_FLOAT)
#define RSF(n,a)   _registry->RegisterFunction(_registry,#n,a,DLL_BASIC4GL_STRING)
#define RTF(n,a,v) _registry->RegisterStructureFunction(_registry,#n,a,v)
#define RVF(n,a)   _registry->RegisterVoidFunction(_registry,#n,a)

' register int,float,string array function
#define RAIF(n,a,d) _registry->RegisterArrayFunction(_registry,#n,a,DLL_BASIC4GL_INT,d)
#define RAFF(n,a,d) _registry->RegisterArrayFunction(_registry,#n,a,DLL_BASIC4GL_FLOAT,d)
#define RAFS(n,a,d) _registry->RegisterArrayFunction(_registry,#n,a,DLL_BASIC4GL_STRING,d)

' register int,float,string constant
#define RIC(n,v) _registry->RegisterIntConstant(_registry,#n,v)
#define RFC(n,v) _registry->RegisterFloatConstant(_registry,#n,v)
#define RSC(n,v) _registry->RegisterStringConstant(_registry,#n,v)

' hType = register type by name,minor,major
#define RT(n,ma,mi) _registry->RegisterStructure(_registry,#n,ma,mi)

' register type padding,byte,word,int,float,string field
#define RTPF(v) _registry->AddStrucPadding(_registry,v) ' nBytes
#define RTBF(n) _registry->AddStrucField(_registry,#n,DLL_BASIC4GL_EXT_BYTE)
#define RTWF(n) _registry->AddStrucField(_registry,#n,DLL_BASIC4GL_EXT_WORD)
#define RTIF(n) _registry->AddStrucField(_registry,#n,DLL_BASIC4GL_EXT_INT)
#define RTFF(n) _registry->AddStrucField(_registry,#n,DLL_BASIC4GL_EXT_FLOAT)
#define RTSF(n,size) _registry->AddStrucStringField(_registry,#n,size) ' size included '0'
' register predefined type as type field
#define RTTF(n,v) _registry->AddStrucField(_registry,#n,v)

' modify last type field as pointer or pointer on pointer
#define MTFLP()   _registry->ModStrucFieldPointer(_registry,1) ' level 1
#define MTFLPLP() _registry->ModStrucFieldPointer(_registry,2) ' level 2

' register byte,word,int,float,string pointer field
#macro RTBPF(n)
  RTBF(n)   ' register byte field by name
  MTFLP()   ' modify as pointer
#endmacro
#macro RTWPF(n)
  RTWF(n)   ' register word field by name
  MTFLP()   ' modify as pointer
#endmacro
#macro RTIPF(n)
  RTIF(n)   ' register int field by name
  MTFLP()   ' modify as pointer
#endmacro
#macro RTFPF(n)
  RTIF(n)   ' register float field by name
  MTFLP()   ' modify as pointer
#endmacro
#macro RTSPF(n,v)
  RTSF(n,v) ' register string field by name and size
  MTFLP()   ' modify as pointer
#endmacro

' register int,float,string pointer on pointer field
#macro RTIPPF(n)
  RTIF(n)   ' register int field by name
  MTFLPLP() ' modify as pointer on pointer
#endmacro
#macro RTFPPF(n)
  RTIF(n)   ' register float field by name
  MTFLPLP() ' modify as pointer on pointer
#endmacro
#macro RTSPPF(n,v)
  RTSF(n,v) ' register string field by name and size
  MTFLPLP() ' modify as pointer on pointer
#endmacro

' modify last type field as 1D,2D or 3D array
#define MTF1D(v1)       _registry->ModStrucFieldArray(_registry,1,v1)
#define MTF2D(v1,v2)    _registry->ModStrucFieldArray(_registry,2,v1,v2)
#define MTF3D(v1,v2,v3) _registry->ModStrucFieldArray(_registry,3,v1,v2,v3)

' register type byte,word,int,float,string array field 1D
#macro RTBA1DF(n,v)
  RTBF(n)    ' register type byte field by name
  MTF1D(v)   ' modify as 1D array
#endmacro
#macro RTWA1DF(n,v)
  RTWF(n)    ' register type word field by name
  MTF1D(v)   ' modify as 1D array
#endmacro
#macro RTIA1DF(n,v)
  RTIF(n)    ' register type int field by name
  MTF1D(v)   ' modify as 1D array
#endmacro
#macro RTFA1DF(n,v)
  RTFF(n)    ' register type float field by name
  MTF1D(v)   ' modify as 1D array
#endmacro
#macro RTSA1DF(n,v0,v1)
  RTSF(n,v0) ' register type string field by name and size
  MTF1D(v1)  ' modify as 1D array
#endmacro

' register type byte,word,int,float,string array field 2D
#macro RTBA2DF(n,v1,v2)
  RTBF(n)      ' register type byte field by name
  MTF2D(v1,v2) ' modify as 2D array
#endmacro
#macro RTWA2DF(n,v1,v2)
  RTWF(n)      ' register type word field by name
  MTF2D(v1,v2) ' modify as 2D array
#endmacro
#macro RTIA2DF(n,v1,v2)
  RTIF(n)      ' register type int field by name
  MTF2D(v1,v2) ' modify as 2D array
#endmacro
#macro RTFA2DF(n,v1,v2)
  RTFF(n)      ' register type float field by name
  MTF2D(v1,v2) ' modify as 2D array
#endmacro
#macro RTSA2DF(n,v0,v1,v2)
  RTSF(n,v0)   ' register type string field by name and size
  MTF2D(v1,v2) ' modify as 2D array
#endmacro

' register type byte,word,int,float,string array field 3D
#macro RTBA3DF(n,v1,v2,v3)
  RTBF(n)         ' register type byte field
  MTF3D(v1,v2,v3) ' modify as 3D array
#endmacro
#macro RTWA3DF(n,v1,v2,v3)
  RTWF(n)         ' register type word field
  MTF3D(v1,v2,v3) ' modify as 3D array
#endmacro
#macro RTIA3DF(n,v1,v2,v3)
  RTIF(n)         ' register type int field
  MTF3D(v1,v2,v3) ' modify as 3D array
#endmacro
#macro RTFA3DF(n,v1,v2,v3)
  RTFF(n)         ' register type float field by name
  MTF3D(v1,v2,v3) ' modify as 3D array
#endmacro
#macro RTSA3DF(n,v0,v1,v2)
  RTSF(n,v0)         ' register type string field by name and size
  MTF3D(v0,v1,v2,v3) ' modify as 3D array
#endmacro

' add int,float,string,struct param
#define AIP _registry->AddParam(_registry,DLL_BASIC4GL_INT)
#define AFP _registry->AddParam(_registry,DLL_BASIC4GL_FLOAT)
#define ASP _registry->AddParam(_registry,DLL_BASIC4GL_STRING)
#define ATP(h) _registry->AddStrucParam(_registry,h)

' add int,float,string,struct param as pointer
#define AIPP _registry->AddParam(_registry,DLL_BASIC4GL_INT):_registry->ModParamPointer(_registry,1)
#define AFPP _registry->AddParam(_registry,DLL_BASIC4GL_FLOAT):_registry->ModParamPointer(_registry,1)
#define ASPP _registry->AddParam(_registry,DLL_BASIC4GL_STRING):_registry->ModParamPointer(_registry,1)
#macro ATPP(h)
  _registry->AddStrucParam(_registry,h)
  _registry->ModParamPointer(_registry,1)
#endmacro

' register int,float,string param as reference
#define AIPR _registry->AddParam(_registry,DLL_BASIC4GL_INT  ):_registry->ModParamReference(_registry)
#define AFPR _registry->AddParam(_registry,DLL_BASIC4GL_FLOAT):_registry->ModParamReference(_registry)
#define ASPR _registry->AddParam(_registry,DLL_BASIC4GL_STRING):_registry->ModParamReference(_registry)
#define ATPR(h) _registry->AddStrucParam(_registry,h):_registry->ModParamReference(_registry)
  
' register int,float,string array param by dimensions
#define AAIP(d) _registry->AddArrayParam(_registry,DLL_BASIC4GL_INT   ,d)
#define AAFP(d) _registry->AddArrayParam(_registry,DLL_BASIC4GL_FLOAT ,d)
#define AASP(d) _registry->AddArrayParam(_registry,DLL_BASIC4GL_STRING,d)

' ###########
' # RUNTIME #
' ###########

' set int,float,string, result
#define SIR(v) _runtime->SetIntResult(_runtime,v)
#define SFR(v) _runtime->SetFloatResult(_runtime,v)
#define SSR(v) _runtime->SetStringResult(_runtime,v)

' set int,float array result
#define SIAR(lpArray,items) _runtime->SetIntArrayResult(_runtime,lpArray,1,items)
#define SFAR(lpArray,items) _runtime->SetFloatArrayResult(_runtime,lpArray,1,items)

' set type return value by handel
#macro STV(t,v)
  _runtime->SetType(_runtime,t)
  _runtime->SetReturnValue(_runtime,v)
#endmacro

' get int,float,string param by index
#define GIP(n) _runtime->GetIntParam(_runtime,n)
#define GFP(n) _runtime->GetFloatParam(_runtime,n)
#define GSP(n) _runtime->GetStringParam(_runtime,n)
' get struct params param by registered hType,nIndex,@Value
#macro GTP(h,n,v) 
  _runtime->SetType(_runtime,h)
  _runtime->GetParam(_runtime,n,v)
#endmacro

' set byref byte,word,int,float,string param
#macro SBPR(n,lpValue)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_BYTE)
  _runtime->ModTypeReference(_runtime)
  _runtime->SetParam(_runtime,n,lpValue)
#endmacro
#macro SWPR(n,lpValue)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_WORD)
  _runtime->ModTypeReference(_runtime)
  _runtime->SetParam(_runtime,n,lpValue)
#endmacro
#macro SIPR(n,lpValue)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_INT)
  _runtime->ModTypeReference(_runtime)
  _runtime->SetParam(_runtime,n,lpValue)
#endmacro
#macro SFPR(n,lpValue)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_FLOAT)
  _runtime->ModTypeReference(_runtime)
  _runtime->SetParam(_runtime,n,lpValue)
#endmacro
#macro SSPR(n,Size,lpValue)
  _runtime->SetStringType(_runtime,Size)
  _runtime->ModTypeReference(_runtime)
  _runtime->SetParam(_runtime,n,lpValue)
#endmacro

' set struct params by registered hType nIndex and @Value
#macro STP(hType,n,lpValue) 
  _runtime->SetType(_runtime,hType)
  _runtime->SetParam(_runtime,n,lpValue)
#endmacro


' get byref byte,word,int,float param
#macro GBPR(n,lpValue)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_BYTE)
  _runtime->ModTypeReference(_runtime)
  _runtime->GetParam(_runtime,n,lpValue)
#endmacro
#macro GWPR(n,v)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_WORD)
  _runtime->ModTypeReference(_runtime)
  _runtime->GetParam(_runtime,n,lpValue)
#endmacro
#macro GIPR(n,lpValue)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_INT)
  _runtime->ModTypeReference(_runtime)
  _runtime->GetParam(_runtime,n,lpValue)
#endmacro
#macro GFPR(n,lpValue)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_FLOAT)
  _runtime->ModTypeReference(_runtime)
  _runtime->GetParam(_runtime,n,lpValue)
#endmacro
' param#,size,@v
#macro GSPR(n,Size,lpValue)
  _runtime->SetStringType(_runtime,Size)
  _runtime->ModTypeReference(_runtime)
  _runtime->GetParam(_runtime,n,lpValue)
#endmacro

' get struct params by registered hType nIndex and @Value
#macro GSTP(h,n,lpValue) 
  _runtime->SetType(_runtime,hType)
  _runtime->GetParam(_runtime,n,lpValue)
#endmacro



  

' get int,float,string array param
#define GIAP _runtime->GetIntArrayParam
#define GFAP _runtime->GetFloatArrayParam
'#define GSAP _runtime->GetStringArrayParam

' modify byte,word,int,float param as array
#macro MBPA(n)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_BYTE)
  _runtime->ModTypeArray(_runtime,n,_runtime->GetArrayParamDimension(_runtime,n,0))
#endmacro
#macro MWPA(n)
  _runtime->SetType(_runtime,DLL_BASIC4GL_WORD_INT)
  _runtime->ModTypeArray(_runtime,n,_runtime->GetArrayParamDimension(_runtime,n,0))
#endmacro
#macro MIPA(n)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_INT)
  _runtime->ModTypeArray(_runtime,n,_runtime->GetArrayParamDimension(_runtime,n,0))
#endmacro
#macro MFPA(n)
  _runtime->SetType(_runtime,DLL_BASIC4GL_EXT_FLOAT)
  _runtime->ModTypeArray(_runtime,n,_runtime->GetArrayParamDimension(_runtime,n,0))
#endmacro

extern "C"
type IDLL_Basic4GL_ObjectStore
  Add           as function (THIS_OBJECTSTORE, object as any ptr) as Integer
  Remove        as sub      (THIS_OBJECTSTORE, handle as integer)
  HandleIsValid as function (THIS_OBJECTSTORE, handle as integer) as integer
  Object        as function (THIS_OBJECTSTORE, handle as integer) as any ptr
  Clr           as sub      (THIS_OBJECTSTORE)
end type
end extern


#define IB4GLCOMPILER_MAJOR 1
#define IB4GLCOMPILER_MINOR 0


' IB4GLCompiler
'
' Used to compile and execute Basic4GL code at RUNTIME.
' IMPORTANT:
'  1. Plugins should ONLY compile code while a program is running
'     (i.e. from within a runtime function.)
'  2. When the program ends the plugin should consider all handles from
'     compiled blocks of code become INVALID. The plugin should not try to
'     call them.
extern "C"
type IB4GLCompiler
  ' Compile Basic4GL code and return its handle.
  ' Returns a non-zero handle if compilation succeeded.
  ' Returns 0 if an error occurs.
  ' virtual int DLLFUNC Compile(const char *sourceText) = 0;
  Compile as function (THIS_COMPILER, _
                       byval SourceText as zstring ptr) as Integer
   

  ' Error retrieving methods. Call only if Compile() returns 0.
  ' Get error description.
  ' Error is placed in buffer.
  ' If error is longer than bufferLen, it will be truncated to fit.
  ' virtual void DLLFUNC GetErrorText(char *buffer, int bufferLen) = 0;
  GetErrorText as sub (THIS_COMPILER, _
                       byval buffer    as zstring ptr, _
                       byval bufferLen as integer)
  ' Line number where error occurred
  ' virtual int DLLFUNC GetErrorLine() = 0;
  GetErrorLine as function(THIS_COMPILER) as integer
  ' Column number where error occurred
  ' virtual int DLLFUNC GetErrorCol() = 0;
  GetErrorCol as function(THIS_COMPILER) as integer
  ' Execute code.
  ' Returns true if code executed without a runtime error.
  ' Returns false if an error occurred. If an error occurs, the plugin
  ' should return immediately. Basic4GL will then stop and display the error.
  '  virtual bool DLLFUNC Execute(int codeHandle) = 0;
  Execute as function (THIS_COMPILER, _
                       byval CodeHandle as integer) as integer
end type
end extern

extern "C"
type IDLL_Basic4GL_Runtime
  GetIntParam      as function (THIS_RUNTIME,index as integer) as integer
  GetFloatParam    as function (THIS_RUNTIME,index as integer) as single
  GetStringParam   as function (THIS_RUNTIME,index as integer) as zstring ptr

  SetIntResult     as sub      (THIS_RUNTIME,result as integer)
  SetFloatResult   as sub      (THIS_RUNTIME,result as single)
  SetStringResult  as sub      (THIS_RUNTIME,result as zstring ptr)

  GetIntArrayParam as sub      (THIS_RUNTIME, _
                                index          as integer, _
                                array          as integer ptr, _
                                dimensions     as integer, _
                                dimension0Size as integer, _
                                ...)
  GetFloatArrayParam  as sub   (THIS_RUNTIME, _
                                index          as integer, _
                                array          as single ptr, _
                                dimensions     as integer, _
                                dimension0Size as integer, _
                                ...)


  ' Return the size of an array dimension.
  ' "index" is the parameter index.
  ' "dimension" is the dimension index, 0 origin.
  ' The return value is the actual number of elements in the array (for that dimension).
  ' Need to keep in mind that the BASIC dim command actually allocates one more element
  ' than the number specified (0..N inclusive).
  ' So if an array is DIMmed as: dim a(10)
  ' this method will return 11 as the number of elements.
  GetArrayParamDimension as function (THIS_RUNTIME, _
                                      index          as integer, _
                                      dimension      as integer) as integer
  ' Set array parameter result
  SetIntArrayResult      as sub      (THIS_RUNTIME, _
                                      array          as integer ptr, _
                                      dimension      as integer, _
                                      dimension0size as integer, _
                                      ...)

  SetFloatArrayResult    as sub      (THIS_RUNTIME, _
                                      array          as single ptr, _
                                      dimension      as integer, _
                                      dimension0size as integer, _
                                      ...)
  
  SetType       as sub (THIS_RUNTIME,typ as integer)
  ' Number of characters (including terminating 0)
  SetStringType as sub (THIS_RUNTIME,size as integer)
  ' Convert a type into an array
  ModTypeArray  as sub (THIS_RUNTIME, _
                        dimensions     as integer, _
                        dimension0Size as integer, _
                        ...)

  ' Convert a type into a reference.
  ' Useful for accessing reference parameters.
  ' Only required if the parameter is a reference to a basic type (int, float or string).
  ' Basic4GL automatically treats arrays and structures as by-reference types, and does
  ' not need to be told!
  ModTypeReference as sub (THIS_RUNTIME)

  ' Reading/writing parameters and return values
  GetParam as sub (THIS_RUNTIME, _
                   index as integer , _ ' Parameter index
                   des   as any ptr)    ' Data is copied here

  SetParam as sub (THIS_RUNTIME, _
                   index as integer, _ ' Parameter index
                   des   as any ptr)   ' Data is copied from here

  SetReturnValue as sub (THIS_RUNTIME,src as any ptr)     ' Data is copied from here

' Get int value at address
  DirectGetInt as function     (THIS_RUNTIME,memAddr as integer) as integer

' Get float value at address
  DirectGetFloat as function   (THIS_RUNTIME,memAddr as integer) as single

  ' Get string value at address.
  ' 'str' and 'maxlen' describe the buffer into which the string will be 
  ' copied (if it is longer than maxLen-1, it will be truncated).
  ' The method always returns 'buffer'
  DirectGetString as function  (THIS_RUNTIME, _ 
                                memAddr as integer, _
                                lpStr   as any ptr, _
                                maxLen  as integer) as any ptr

  ' Write int value to address
  DirectSetInt as sub          (THIS_RUNTIME, _
                                memAddr as integer, _
                                value   as integer)

  ' Write float value to address
  DirectSetFloat as sub        (THIS_RUNTIME, _
                                memAddr as integer, _
                                value   as single)

  ' Write string value to address
  DirectSetString as sub       (THIS_RUNTIME, _ 
                                memAddr as integer, _
                                src     as any ptr )
end type
end extern
type DLL_Basic4GL_Function as sub CDECL (THIS_RUNTIME)

extern "C"
type IDLL_Basic4GL_ObjectStoreCallback
  ObjectRemoved as sub (THIS_OBJECTSTORE, object as any ptr)
end type
end extern

extern "C"
type IDLL_Basic4GL_FunctionRegistry
  ' Register a constant
  RegisterStringConstant as sub  (THIS_REGISTRY, _
                                  nam     as zstring ptr, _
                                  value   as zstring ptr)

  RegisterIntConstant    as sub  (THIS_REGISTRY, _
                                  nam     as zstring ptr, _
                                  value   as integer)

  RegisterFloatConstant  as sub  (THIS_REGISTRY, _
                                  nam     as zstring ptr, _
                                  value    as single)

  ' Register function that returns no value
  RegisterVoidFunction   as sub  (THIS_REGISTRY, _
                                  nam     as zstring ptr, _
                                  func    as DLL_Basic4GL_Function)

  ' Register function that returns single value
  RegisterFunction       as sub  (THIS_REGISTRY, _
                                  nam      as zstring ptr, _
                                  func     as DLL_Basic4GL_Function, _
                                  typecode as DLL_Basic4GL_TypeCode)

  ' Register function that returns an array
  RegisterArrayFunction  as sub   (THIS_REGISTRY, _
                                   nam       as zstring ptr, _
                                   func      as DLL_Basic4GL_Function, _
                                   typecode  as DLL_Basic4GL_TypeCode, _
                                   dimension as integer)

  ' Register function that returns a structure.
  ' hStruc is the handle returned from RegisterStructure(), or FetchStructure().
  RegisterStructureFunction as sub (THIS_REGISTRY, _
                                    nam    as zstring ptr, _
                                    func   as DLL_Basic4GL_Function, _
                                    hStruc as integer)



  ' Function modifiers
  ' Modifies the last registered function to return an array.
  ' E.g. if the function was registered as returning an integer, this will
  ' alter it to return an array of integers.
  ModReturnArray as sub            (THIS_REGISTRY, _
                                    dimensions as integer)

  ' Modifies the last registered function to return a pointer.
  ' E.g. if the function was registered as returning a string, this will
  ' alter it to return a pointer to a string.
  ' Note:
  ' Returning pointers is an advanced operation, and you need to be 
  ' careful that you understand Basic4GL's data formats and know exactly 
  ' what you're doing. Unless you know exactly why you need to return 
  ' a pointer from a DLL function, you should avoid this method.
  '  Level is the level of indirection:
  ' 1 = Pointer (most common)
  ' 2 = Pointer to pointer
  ModReturnPointer as sub          (THIS_REGISTRY, _
                                    level as integer)


  ' Modifies the last registered function to have no brackets.
  ModNoBrackets as sub             (THIS_REGISTRY)

  ' Modifies the last registered function to trigger a timesharing break after it is called.
  ModTimeshare as sub              (THIS_REGISTRY)

  ' Add standard parameter to the last registered function.
  ' Parameters should be added from left to right.
  ' Please be aware that the RIGHTMOST parameter will have index 1 when the
  ' function is called at runtime, i.e. the LAST parameter added.
  AddParam as sub                  (THIS_REGISTRY, _
                                    typeCode as DLL_Basic4GL_TypeCode)

    ' Add an array parameter to the last registered function.
  AddArrayParam as sub             (THIS_REGISTRY, _
                                    typeCode   as DLL_Basic4GL_TypeCode, _
                                    dimensions as integer)

  ' Add a structure parameter to the last registered function.
  ' handle' is the structure type handle returned from "RegisterStructure" or "FetchStructure"
  AddStrucParam as sub             (THIS_REGISTRY, _ 
                                    hStruc as integer)

  ' Convert the last parameter added into an array parameter.
  ModParamArray as sub             (THIS_REGISTRY, _ 
                                    dimensions as integer)

  ' Convert the last parameter added into a pointer.
  ' 'level' is the level of indirection:
  ' Using pointers is an advanced function, and you need to be sure you know
  ' what you're doing. If in doubt, avoid...
  ' 1 = Pointer (most common)
  ' 2 = Pointer to pointer etc
  ModParamPointer as sub           (THIS_REGISTRY, _ 
                                    level as integer)

  ' Convert the last parameter added into a reference parameter.
  ' This is like Pascal's "var" parameters, or C++'s & parameters.
  ' They can be used to return values.
  ' Note: 
  ' The parameter will appear as a pointer to the 
  ' plugin function, and should be accessed like one.
  ' Note 2: 
  ' Structure and array parameters are automatically passed by 
  ' reference, so you don't need to call this method for them.
  ModParamReference as sub         (THIS_REGISTRY)

  ' Register a new structure.
  ' Returns a handle allowing the structure to be referenced 
  ' in function or parameter declarations.
  ' return 0 if fails can fail if the structure has been registered already.
  ' Plugin should set an appropriate error and return false.
  RegisterStructure as function (THIS_REGISTRY, _ 
                                 nam   as zstring ptr, _
                                 Major as integer    , _
                                 Minor as integer) as integer

  ' Add empty padding field
  AddStrucPadding as sub (THIS_REGISTRY, numBytes as integer)

  ' Add regular field.
  ' type can be a DLL_Basic4GL_Ext_TypeCode constant, or structure handle.
  AddStrucField as sub (THIS_REGISTRY, nam as zstring ptr, typ as integer)

  ' Add a string field.
  ' Must supply string size in bytes (including 0 terminator)
  AddStrucStringField as sub (THIS_REGISTRY, nam as zstring ptr, size as integer)

  ' Convert the last defined structure field into an array
  ModStrucFieldArray as sub (THIS_REGISTRY, _ 
                             dimensions     as integer, _
                             dimension1Size as integer, _
                             ...) = 0

  ' Convert the last defined structure field into a pointer
  ' Note: 'level' = the level of indirection.
  ' 1 = Pointer (most common)
  ' 2 = Pointer to pointer
  ModStrucFieldPointer as sub (THIS_REGISTRY, level as integer)

  ' Fetch structure (presumably registered by another plugin).
  ' Returns handle, or 0 if not registered.
  FetchStructure as function (THIS_REGISTRY, _ 
                              nam    as zstring ptr, _
                              vMajor as integer    , _
                              vMinor as integer) as integer

  RegisterInterface as sub (THIS_REGISTRY, _
                            object as any ptr, _
                            nam    as zstring ptr, _
                            major  as integer, _
                            minor  as integer)

  FetchInterface as function (THIS_REGISTRY, _
                              nam   as zstring ptr, _
                              major as integer, _
                              minor as integer) as any ptr
  CreateObjectStore as function (THIS_REGISTRY, _
                                 byref Callback as IDLL_Basic4GL_ObjectStoreCallback ptr) _
                                 as IDLL_Basic4GL_ObjectStore ptr ptr
end type
end extern
'type LPIDLL_Basic4GL_FunctionRegistry as IDLL_Basic4GL_FunctionRegistry ptr

extern "C"
type IDLL_Basic4GL_Plugin
  Load            as function (THIS_PLUGIN, _
                               THIS_REGISTRY, _
                               blnIsExe as byte) as integer
  Unload          as sub      (THIS_PLUGIN)
  Start           as function (THIS_PLUGIN) as integer
  Ends            as sub      (THIS_PLUGIN)
  Pause           as sub      (THIS_PLUGIN) 
  Resumed         as sub      (THIS_PLUGIN)
  DelayedResume   as sub      (THIS_PLUGIN)
  GetError        as sub      (THIS_PLUGIN, _
                               ErrMsg as zstring ptr)
  ProcessMessages as sub      (THIS_PLUGIN)
end type
end extern

#endif ' __BASIC4GLPLUGIN__